'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
const vscode = require("vscode");
var net = require('net');
//var fs = require('fs');
var client = undefined;
var InstanceId = Math.random().toString(36).replace(/[^a-z]+/g, "").substr(0, 5);
var data = {};
var port;
var lineReader;
var lines = [];
var outChannel;
//var ignoreSaveNotification = false;
data["Id"] = 'VS+' + InstanceId;
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    // Use the console to output diagnostic information (console.log) and errors (console.error)
    // This line of code will only be executed once when your extension is activated
    console.log('Congratulations, your extension "calcext" is now active!');
    outChannel = vscode.window.createOutputChannel("DEBUG");
    try {
        if (vscode.workspace.workspaceFolders !== undefined) {
            //var path = "\"" + vscode.workspace.workspaceFolders[0].uri.fsPath + '\\.stf.' + "\"";
            var path = vscode.workspace.workspaceFolders[0].uri.fsPath + '\\.stf';
            //var path = ".stf."
            //var path = vscode.workspace.workspaceFolders[0].uri.path + '/.stf';
            lineReader = require('readline').createInterface({
                input: require('fs').createReadStream(path)
            });
        }
        lineReader.on('line', function (line) {
            console.log('Line from file:', line);
            lines.push(line);
        });
        lineReader.on('close', function () {
            console.log('All lines read!');
            if (lines.length > 0 && lines[0].length > 0) {
                port = Number(lines[0]);
            }
            console.log('creating Socket');
            //client = new net.Socket();
            console.log('trying to connect');
            //client.connect(port, '127.0.0.1', function() {
            client = net.createConnection({ port: port }, () => {
                console.log('Client connected...');
                //vscode.commands.executeCommand('workbench.debug.action.toggleRepl');
                vscode.commands.executeCommand('workbench.action.output.toggleOutput');
                //vscode.debug.activeDebugConsole.appendLine("CONNECTED!");
                outChannel.appendLine("CONNECTED!");
                outChannel.show();
                data["Command"] = "Connect";
                if (vscode.workspace.workspaceFolders !== undefined) {
                    data["FileName"] = vscode.workspace.workspaceFolders[0].uri.fsPath; //workspaceFolders[0].uri.fsPath;
                }
                client.write(MessageToBuffer(JSON.stringify(data)));
            });
            client.on('error', function (e) {
                console.log(e.message);
                //vscode.debug.activeDebugConsole.appendLine("ERROR: " + e.message );
                outChannel.appendLine("ERROR: " + e.message);
                outChannel.show();
            });
            client.on('close', function () {
                console.log('Connection closed');
                //vscode.debug.activeDebugConsole.appendLine("Connection Closed..." );
                outChannel.appendLine("Connection Closed...");
                outChannel.show();
            });
            client.on('data', function (buf) {
                console.log('Data received...');
                console.log(buf);
                var length = buf.readInt32LE();
                var buffer = Buffer.alloc(length);
                buf.copy(buffer, 0, 4);
                var str = buffer.toString();
                var json = JSON.parse(str);
                if (json["Command"] === "QueryDocDirty") {
                    //vscode.debug.activeDebugConsole.appendLine("Command: QueryDocDirty from Server");
                    var status = "false";
                    for (var i = 0; i < vscode.workspace.textDocuments.length; ++i) {
                        if (vscode.workspace.textDocuments[i].fileName.includes(json["FileName"])) {
                            if (vscode.workspace.textDocuments[i].isDirty === true) {
                                status = "true";
                                break;
                            }
                        }
                    }
                    data["Command"] = "QueryDocDirty";
                    data["Status"] = status;
                    client.write(MessageToBuffer(JSON.stringify(data)));
                }
                else if (json["Command"] === "QueryAnyDocDirty") {
                    //vscode.debug.activeDebugConsole.appendLine("Command: QueryDocDirty from Server");
                    var status = "false";
                    for (var i = 0; i < vscode.workspace.textDocuments.length; ++i) {
                        if (vscode.workspace.textDocuments[i].isDirty === true) {
                            status = "true";
                            break;
                        }
                    }
                    data["Command"] = "QueryAnyDocDirty";
                    data["Status"] = status;
                    client.write(MessageToBuffer(JSON.stringify(data)));
                }
                else if (json["Command"] === "SaveAll") {
                    vscode.workspace.saveAll(false);
                }
                else if (json["Command"] === "RevertChanges") {
                    for (var i = 0; i < vscode.workspace.textDocuments.length; ++i) {
                        vscode.window.showTextDocument(vscode.workspace.textDocuments[i]);
                        vscode.commands.executeCommand('workbench.action.files.revert');
                    }
                }
                else if (json["Command"] === "CloseEditor") {
                    vscode.commands.executeCommand('workbench.action.closeWindow');
                }
                else if (json["Command"] === "CloseWorkspace") {
                    vscode.commands.executeCommand("workbench.action.closeEditorsInGroup");
                    vscode.workspace.updateWorkspaceFolders(0, 1);
                }
                else if (json["Command"] === "OpenWorkspace") {
                    vscode.workspace.updateWorkspaceFolders(0, 1, { uri: vscode.Uri.file(json["FileName"]) });
                }
                else if (json["Command"] === "ShowDocument") {
                    if (vscode.window.activeTextEditor === undefined || vscode.window.activeTextEditor != undefined && vscode.window.activeTextEditor.document.fileName.toLowerCase() !== json["FileName"].toLowerCase()) {
                        vscode.workspace.openTextDocument(json["FileName"]).then(doc => {
                            vscode.window.showTextDocument(doc);
                        }, (err) => {
                            vscode.window.showErrorMessage("Cannot display: " + json["FileName"]);
                        });
                    }
                    else if (vscode.window.activeTextEditor != undefined && vscode.window.activeTextEditor.document.fileName.toLowerCase() === json["FileName"].toLowerCase()) {
                        data["Command"] = "OpenDoc";
                        data["FileName"] = json["FileName"];
                        client.write(MessageToBuffer(JSON.stringify(data)));
                    }
                }
                else if (json["Command"] === "FoldAllRegions") {
                    vscode.commands.executeCommand("editor.foldAllMarkerRegions");
                }
                else if (json["Command"] === "Ping") {
                    data["Command"] = "Pong";
                    data["Status"] = "";
                    client.write(MessageToBuffer(JSON.stringify(data)));
                }
            });
        });
    }
    catch (err) {
    }
    // The command has been defined in the package.json file
    // Now provide the implementation of the command with  registerCommand
    // The commandId parameter must match the command field in package.json
    vscode.workspace.onDidOpenTextDocument((document) => {
        //console.log("onDidOpenTextDocument: " + document.fileName);
        //vscode.debug.activeDebugConsole.appendLine("onDidOpenTextDocument: " + document.fileName );
        data["Command"] = "OpenDoc";
        data["FileName"] = document.fileName;
        client.write(MessageToBuffer(JSON.stringify(data)));
    });
    vscode.workspace.onDidSaveTextDocument((document) => {
        // console.log("onDidSaveTextDocument");
        //vscode.debug.activeDebugConsole.appendLine("onDidSaveTextDocument: " + document.fileName );
        data["Command"] = "SaveDoc";
        data["FileName"] = document.fileName;
        client.write(MessageToBuffer(JSON.stringify(data)));
    });
    vscode.workspace.onDidCloseTextDocument((document) => {
        //console.log("onDidCloseTextDocument: " + document.fileName);
        //vscode.debug.activeDebugConsole.appendLine("onDidCloseTextDocument: " + document.fileName );
        data["Command"] = "CloseDoc";
        data["FileName"] = document.fileName;
        client.write(MessageToBuffer(JSON.stringify(data)));
    });
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() {
    //console.log('Your extension is now deactivated!');
    data["Command"] = "Disconnecting";
    data["FileName"] = "";
    client.write(MessageToBuffer(JSON.stringify(data))).then(() => {
        client.close();
    }, () => {
        console.log('Error on connection close');
    });
    outChannel.dispose();
}
exports.deactivate = deactivate;
function MessageToBuffer(msg) {
    var buffer = Buffer.alloc(msg.length + 4);
    buffer.writeInt32LE(msg.length, 0);
    buffer.write(msg, 4);
    return buffer;
}
//# sourceMappingURL=extension.js.map